<?php

/**
 * Movable Type Backup Importer
 *
 * @package WordPress
 * @subpackage Importer
 */
class MT_Backup_Import extends WP_Importer {
	
	private $_elements = array(
		array('id' => 'author', 'label' => 'Users'),
		array('id' => 'image', 'label' => 'Images'),
		array('id' => 'file', 'label' => 'Files'),
		//array('id' => 'objectasset', 'label' => 'Bildverknüpfungen'),
		array('id' => 'entry', 'label' => 'Posts'),
		array('id' => 'tag', 'label' => 'Tags'),
		//array('id' => 'objecttag', 'label' => 'Tags'),
		array('id' => 'category', 'label' => 'Categories'),
		//array('id' => 'placement', 'label' => 'Categories'),
		array('id' => 'comment', 'label' => 'Comments'),
		array('id' => 'trackback', 'label' => 'Trackbacks')
	);
	
	private $_siteurl;
	
	function MT_Backup_Import() {
		// NOOP
	}
	
	function header() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>'.__('Import Movable Type Backup', PLUGIN_ID).'</h2>';
	}

	function footer() {
		echo '</div>';
	}

	function greet() {
		$this->header();
		$directory = wp_upload_dir();
		$basedir = $directory['basedir'];
		$backups = glob($basedir . '/import/Movable_Type-*-Backup*.xml');
		
		if (sizeof($backups) > 0) {
		?>
		<h3>Choose Backup to Import</h3>
		<div class="narrow">
		<p><?php _e('Nice! There are backups in the correct directory. Let\'s start by choosing a backup to import.') ?></p>
		<form method="post" action="<?php echo esc_attr(add_query_arg('step', 1)); ?>">
		<?php wp_nonce_field('import-mt-backup'); ?>
		<p>
		 <ul>
		  <?php foreach($backups as $index => $backup) { ?>
		  <li>
		   <input type="checkbox" id="backup-<?php echo $index; ?>" <?php if($index == 0) { ?>checked="checked"<?php } ?> name="backup" value="<?php echo basename($backup) ?>" />
		   <label for="backup-<?php echo $index; ?>"><i><?php echo basename($backup) ?></i></label>
		  </li>
		  <?php } ?>
		 </ul>
		</p>
		<h3><?php _e('Which objects would you like to import?') ?></h3>
		<p>
		 <ul>
		  <?php foreach($this->_elements as $index => $element) { ?>
		  <li>
		   <input checked="checked" type="checkbox" disabled="disabled" id="element-<?php echo $index; ?>" name="elements[<?php echo $index ?>]" value="<?php echo $element['id'] ?>" />
		   <label for="element-<?php echo $index; ?>"><i><?php echo $element['label'] ?></i></label>
		  </li>
		  <?php } ?>
		 </ul>
		</p>
		<p class="submit">
		<input type="submit" class="button" value="<?php esc_attr_e('Import', PLUGIN_ID); ?>" />
		</p>
		</form>
		</div>
		
		<?php } else { ?>
		<h3>Upload Backup</h3>
		<div class="narrow">
		<p>
		 <ul>
		  <li>1. <?php _e('Upload <b>unzipped</b> backup to') ?>: <code><?php echo $basedir ?>/import</code></li>
		  <li>2. <?php _e('Change permission of this directory to') ?> <code>777</code> (or just writeable by the apache user)</li>
		 </ul>
	    </p>
		<form method="post" action="<?php echo esc_attr(add_query_arg('step', 0)); ?>">
		<p class="submit">
		<input type="submit" class="button" value="<?php esc_attr_e('Refresh', PLUGIN_ID); ?>" />
		</p>
		</form>
		</div>
		<?php
		}
		$this->footer();
	}
		
	function dispatch() {
		$step = empty($_GET['step']) ? 0 : (int) $_GET['step'];
		switch ($step) {
			case 0 :
				$this->greet();
				break;
			case 1:
				check_admin_referer('import-mt-backup');
				set_time_limit(0);
				$result = $this->import();
				if (is_wp_error($result))
					echo $result->get_error_message();
				break;
		}
	}
	
	function import() {
		echo '<h3>Importing...</h3>';
		$directory = wp_upload_dir();
		$basedir = $directory['basedir'] . '/import/';
		$backup = $_REQUEST['backup'];
		
		// cleanup xml file before import
		$content = file_get_contents($basedir . $backup);
		// remove bad characters
		$content = str_replace(array("", ""), array("",""), $content);
		// remove unused log entries
		$content = preg_replace("%(<log.*?>.*?<\/log.*?>)%is", '', $content);
		// write temporary clean xml file
		file_put_contents($basedir . 'clean-' . $backup, $content);
		unset($content);
		
		// start import
		$this->_import_mt($basedir, 'clean-' . $backup);
	}
	
	function _import_mt($path, $backup) {
		$backupxml = $path . $backup;
		echo '<p>Import <code>' . $backupxml . '</code>.</p>';
		if (is_file($backupxml)) {
			$xml = simplexml_load_file($backupxml);
			$nodes = $xml->children();
			$mappings = array(
				'user' => array(), 
				'post' => array(),
				'assets' => array(),
				'tags' => array(),
				'categories' => array()
			);
			foreach ($nodes as $name => $node) {
				switch ($name) {
					case 'blog': $this->_import_mt_blog($path, $node, $mappings); break;
					case 'author': $this->_import_mt_user($path, $node, $mappings); break;
					case 'image': $this->_import_mt_asset($path, $node, $mappings); break;
					case 'file': $this->_import_mt_asset($path, $node, $mappings); break;
					case 'entry': $this->_import_mt_post($path, $node, $mappings); break;
					case 'tag': $this->_import_mt_tag($path, $node, $mappings); break;
					case 'placement': $this->_import_mt_placement($path, $node, $mappings); break;
					case 'category': $this->_import_mt_category($path, $node, $mappings); break;
					case 'objecttag': $this->_import_mt_objecttag($path, $node, $mappings); break;
					case 'objectasset': $this->_import_mt_objectasset($path, $node, $mappings); break;
					case 'comment': $this->_import_mt_comment($path, $node, $mappings); break;
					// case 'trackback': $this->_import_mt_trackback($path, $node, $mappings); break;
				}
			}
		}
	}
	
	function _import_mt_blog($path, $blog, &$mappings) {
		$this->_siteurl = $blog->attributes()->site_url;
	}

	function _import_mt_user($path, $user, &$mappings) {

		$id = $user->attributes()->id;
		$email = $user->attributes()->email;
		$username = $user->attributes()->name;
		$nickname = $user->attributes()->nickname;
		$status = $user->attributes()->status; // 1 = Active, 2 = Inactive
		$password = $user->attributes()->password;
		$password = strlen($password) > 0 ? $password : wp_generate_password();
		$created = $user->attributes()->created_on;
		$role = $status == 1 ? 'author' : 'subscriber';

		if (substr($username,0,10) == 'ldaprename') {
			$user_id = username_exists(substr($username,10));
			$mappings['user'][(string)$id] = $user_id;
			return;
		}

		$data = array(
			'user_pass' => $password,
			'user_login' => $username,
			'user_email' => $email,
			'display_name' => $nickname,
			'user_nicename' => $nickname,
			'user_registered' => date('Y-m-d H:i:s', strtotime($created)),
			'role' => $role
		);
		$user_id = username_exists($username);
		if (!$user_id) {
			$user_id = wp_insert_user($data);
		}
		
		// add author role to user for this blog
		global $current_blog;
		if (function_exists('add_user_to_blog')) {
			add_user_to_blog($current_blog->blog_id, $user_id, 'editor');
		}
		$mappings['user'][(string)$id] = $user_id;
	}

	function _import_mt_asset($path, $asset, &$mappings) {

		$id = (string) $asset->attributes()->id;
		$filename = (string) $asset->attributes()->file_name;
		$mimetype = (string) $asset->attributes()->mime_type;
		$label = (string) $asset->attributes()->label;
		$parent = (string) $asset->attributes()->parent;
		$entry_id = (string) $asset->attributes()->entry_id;
		$title = strlen($label) > 0 ? $label : $filename;
		$author = (string) $asset->attributes()->created_by;

		// Skip, if it's a thumbnail
		if ($parent > 0) {
			return;
		}

		$attachment = array(
		     'post_mime_type' => $mimetype,
		     'post_title' => preg_replace('/\.[^.]+$/', '', $title),
		     'post_content' => '',
		     'post_status' => 'inherit',
			 'post_author' => isset($mappings['user'][$author]) ? $mappings['user'][$author] : null,
		);

		$filepath = $path . '/' . $id . '-' . $filename;
		$attach_id = wp_insert_attachment($attachment, $filepath, null);
		require_once(ABSPATH . 'wp-admin/includes/image.php');
		$attach_data = wp_generate_attachment_metadata($attach_id, $filepath);
		wp_update_attachment_metadata($attach_id, $attach_data);
		$mappings['assets'][$id] = $attach_id;
	}

	function _import_mt_post($path, $post, &$mappings) {

		$id = $post->attributes()->id;
		$allow_comments = $post->attributes()->allow_comments;
		$allow_pings = $post->attributes()->allow_pings;
		$atom_id = $post->attributes()->atom_id;
		$author_id = (string) $post->attributes()->author_id;
		$authored_on = date('Y-m-d H:i:s', strtotime($post->attributes()->authored_on));
		$basename = $post->attributes()->basename;
		$comment_count = $post->attributes()->comment_count;
		$class = $post->attributes()->class;
		$created_by = $post->attributes()->created_by;
		$created_on = date('Y-m-d H:i:s', strtotime($post->attributes()->created_on));
		$modified_by = $post->attributes()->modified_by;
		$modified_on = date('Y-m-d H:i:s', strtotime($post->attributes()->modified_on));
		$ping_count = $post->attributes()->ping_count;
		$status = $post->attributes()->status;
		$title = $post->attributes()->title;
		$weeknumber = $post->attributes()->weeknumber;
		$content = (string) $post->text;
		$search = array();
		$replace = array();
		$preg_search = array();
		$preg_replace = array();
		
		if (strlen($content) > 0) {
			$doc = new DOMDocument();
			$doc->loadHTML($content);
			$xml = simplexml_import_dom($doc);
			$siteurl_length = strlen($this->_siteurl);
			
			// get all image urls and replace them with the new ones
			$images = $xml->xpath('//img');
			foreach ($images as $img) {
				if (substr($img['src'], 0, $siteurl_length) == $this->_siteurl) {
					$filename = urldecode(basename($img['src']));
					$originals = glob($path . '/*-' . $filename);
					if (sizeof($originals) > 0) {
						$new = basename($originals[0]);
						$search[] = (string) $img['src'];
						$replace[] = get_bloginfo('url') . '/files/import/' . $new;
					}
				}
			}
			
			// get all link urls and replace them with the new ones
			$links = $xml->xpath('//a');
			foreach ($links as $link) {
				if (substr($link['href'], 0, $siteurl_length) == $this->_siteurl) {
					$filename = urldecode(basename($link['href']));
					$originals = glob($path . '/*-' . $filename);
					$original_link = (string) $link['href'];
					$search[] = $original_link;
					if (sizeof($originals) > 0) {
						$new = basename($originals[0]);
						$replace[] = get_bloginfo('url') . '/files/import/' . $new;
					} else {
						$replace[] = str_replace($this->_siteurl, get_bloginfo('url') . '/', $link['href']);
					}
				}
			}
			
			// replace iframes
			$iframes = $xml->xpath('//iframe');
			foreach ($iframes as $iframe) {
				$src = str_replace(array('/', '?', '.'), array('\/', '\?', '\.'), $iframe['src']);
				$preg_search[] = '%<iframe.*?src="' . $src . '".*?>.*?<\/iframe.*?>%is';
				$preg_replace[] = str_replace('http://www.youtube.com/embed/', 'http://www.youtube.com/watch?v=', $iframe['src']);
			}
			
			$pattern = '%<form mt:asset-id.*?><a href="(.*?)">.*?<img.*?src="(.*?)".*?<\/a><\/form>%is';
			preg_match_all($pattern, $content, $matches);
			foreach ($matches[0] as $key => $match) {
				$content = str_replace($match, '<img src="' . $matches[2][$key] . '" alt="' . basename($matches[2][$key]) . '" width="550" />', $content);
			}
			
			$content = str_replace($search, $replace, $content);
			$content = preg_replace($preg_search, $preg_replace, $content);
			
		}


		$post = new StdClass;
		$post->post_content = $content;
		$post->post_author = isset($mappings['user'][$author_id]) ? $mappings['user'][$author_id] : null;
		$post->post_title = $title;
		$post->post_name = $basename;
		$post->post_status = $status == 2 ? 'publish' : null; // TODO
		$post->post_modified = $modified_on;
		$post->post_modified_gmt = get_gmt_from_date($modified_on);
		$post->post_date = $created_on;
		$post->post_date_gmt = get_gmt_from_date($created_on);
		$post->ping_status = $allow_pings == 1 ? true : false;
		$post->comment_status = $allow_comments == 1 ? 'open' : 'closed';
		
		// Tell Wordpress that the content is already filtered to allow embed, iframe, etc.
		// Credits to: http://pp19dd.com/2010/06/unfiltered-wp_insert_post/
		$post->filter = true;

		$post_id = $this->_save_post($post);
		$mappings['posts'][(string)$id] = $post_id;

	}

	function _import_mt_tag($path, $tag, &$mappings) {

		$id = $tag->attributes()->id;
		$n8d_id = $tag->attributes()->n8d_id;
		$name = $tag->attributes()->name;
		$mappings['tags'][(string)$id] = (string)$name;

	}

	function _import_mt_objecttag($path, $tag, &$mappings) {

		$id = $tag->attributes()->id;
		$object_datasource = $tag->attributes()->object_datasource;
		$object_id = (string) $tag->attributes()->object_id;
		$tag_id = (string) $tag->attributes()->tag_id;

		if ($object_datasource == 'entry') {
			wp_add_post_tags($mappings['posts'][$object_id], $mappings['tags'][$tag_id]);
		}

	}

	function _import_mt_category($path, $category, &$mappings) {

		$id = $category->attributes()->id;
		$allow_pings = $category->attributes()->allow_pings;
		$author_id = $category->attributes()->author_id;
		$basename = $category->attributes()->basename;
		$created_by = $category->attributes()->created_by;
		$created_on = $category->attributes()->created_on;
		$label = $category->attributes()->label;
		$modified_on = $category->attributes()->modified_on;
		$parent = $category->attributes()->parent;

		$data = array(
		  'cat_name' => $label,
		  'category_nicename' => $basename
		);
		$category = get_category_by_slug($basename);
		if (!$category) {
			$cat_id = wp_insert_category($data, $wp_error);
		} else {
			$cat_id = $category->cat_ID;
		}
		$mappings['categories'][(string)$id] = $cat_id;

	}

	function _import_mt_placement($path, $placement, &$mappings) {

		$id = $placement->attributes()->id;
		$category_id = (string) $placement->attributes()->category_id;
		$post_id = (string) $placement->attributes()->entry_id;
		$is_primary = $placement->attributes()->is_primary; // 1 = true, 0 = false

		wp_set_post_categories($mappings['posts'][$post_id], array($mappings['categories'][$category_id]));

	}

	function _import_mt_objectasset($path, $objectasset, &$mappings) {

		$id = (string) $objectasset->attributes()->id;
		$asset_id = (string) $objectasset->attributes()->asset_id;
		$embedded = (string) $objectasset->attributes()->embedded;
		$object_ds = (string) $objectasset->attributes()->object_ds;
		$object_id = (string) $objectasset->attributes()->object_id;
		if ($object_ds == 'entry') {
			wp_update_post(array('ID' => $mappings['assets'][$asset_id], 'post_parent' => $mappings['posts'][$object_id]));
		}

	}

	function _import_mt_comment($path, $comment, &$mappings) {

		$id = (string) $comment->attributes()->id;
		$author = (string) $comment->attributes()->author;
		$created_on = date('Y-m-d H:i:s', strtotime($comment->attributes()->created_on));
		$entry_id = (string) $comment->attributes()->entry_id;
		$email = (string) $comment->attributes()->email;
		$url = (string) $comment->attributes()->url;
		$ip = (string) $comment->attributes()->ip;
		$junk_status = (string) $comment->attributes()->junk_status; // 1 = OK
		$last_moved_on = date('Y-m-d H:i:s', strtotime($comment->attributes()->last_moved_on));
		$visible = (string) $comment->attributes()->visible;
		$text = (string) $comment->text;

		$comment = array();
		$comment['comment_author'] = $author;
		$comment['comment_author_email'] = $email;
		$comment['comment_date'] = $created_on;
		$comment['comment_status'] = 'open';
		$comment['comment_author_IP'] = $ip;
		$comment['comment_author_url'] = $url;
		$comment['comment_content'] = $text;

		$comment = add_magic_quotes($comment);
		if (!comment_exists($comment['comment_author'], $comment['comment_date'])) {
			$comment['comment_post_ID'] = $mappings['posts'][$entry_id];
			$comment = wp_filter_comment($comment);
			wp_insert_comment($comment);
		}

	}

	function _import_mt_trackback($path, $trackback, &$mappings) {

		$id = (string) $trackback->attributes()->id;
		$created_by = (string) $trackback->attributes()->created_by;
		$created_on = date('Y-m-d H:i:s', strtotime($trackback->attributes()->created_on));
		$entry_id = (string) $trackback->attributes()->entry_id;
		$title = (string) $trackback->attributes()->email;
		$url = (string) $trackback->attributes()->url;
		$modified_on = date('Y-m-d H:i:s', strtotime($trackback->attributes()->modified_on));
		$is_disabled = (string) $trackback->attributes()->is_disabled;
		$text = (string) $trackback->description;

		$ping = array();
		$ping['comment_type'] = 'trackback';
		$ping['comment_author'] = $mappings['users'][$created_by];
		$ping['comment_date'] = $created_on;
		$ping['ping_status'] = 'open';
		$ping['comment_author_url'] = $url;
		$ping['comment_content'] = "<strong>{$ping['title']}</strong>\n\n{$ping['comment_content']}";

		$ping = add_magic_quotes($ping);
		if (!comment_exists($ping['comment_author'], $ping['comment_date'])) {
			$ping['comment_post_ID'] = $mappings['posts'][$entry_id];
			$ping = wp_filter_comment($ping);
			wp_insert_comment($ping);
		}

	}

	function _save_post(&$post) {
		$post = get_object_vars($post);
		$post = add_magic_quotes($post);
		$post = (object) $post;

		if ( $post_id = post_exists($post->post_title, '', $post->post_date) ) {
			echo '<li>';
			printf(__('Post <em>%s</em> already exists.', PLUGIN_ID), stripslashes($post->post_title));
		} else {
			echo '<li>';
			printf(__('Importing post <em>%s</em>...', PLUGIN_ID), stripslashes($post->post_title));
			$post_id = wp_insert_post($post);
			if (is_wp_error($post_id)) {
				echo "ERROR";
				return $post_id;
			}
		}
		return $post_id;
	}
	
	function _get_blog_list() {
		global $wpdb;
		$blogs = $wpdb->get_results($wpdb->prepare("
			SELECT blog_id, domain, path 
			FROM $wpdb->blogs 
			WHERE 
				public = '1' AND 
				archived = '0' AND 
				mature = '0' AND 
				spam = '0' AND 
				deleted = '0' 
			ORDER BY registered DESC"
			), ARRAY_A
		);
	}
	
}

?>